﻿namespace Phoenix.Main.Frame
{
    using System;
    using System.Reflection;
    using System.Threading;
    using System.Windows.Forms;

    using Phoenix.Log.Writer;
    using Phoenix.Model.Dal.Configuration;
    using Phoenix.Translation;
    using Phoenix.View;
    using Phoenix.View.Helpers;

    using StructureMap;

    public partial class SplashScreen : TranslatedForm
    {
        #region Fields

        private const string LoadMsg = "Loading{0}";
        private const string TitleMsg = "Phoenix BETA {0}";

        private Semaphore semaphore = new Semaphore(1, 1);
        private int step = 0;
        private Thread th_Animation;
        private Thread th_Load;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SplashScreen"/> class.
        /// </summary>
        public SplashScreen()
        {
            base.Logger = LogManager.GetLogger(typeof(SplashScreen));
            InitializeComponent();
            this.Opacity = 0;
            this.IsAlive = true;

            base.Logger = LogManager.GetLogger(typeof(SplashScreen));
            base.Logger.Debug("Splash screen is loaded!");

            this.th_Animation = new Thread(this.ThreadedAnimation);
            this.th_Animation.Name = "Waiting animation thread";

            this.th_Load = new Thread(this.ThreadLoad);
            this.th_Load.Name = "Loading process thread";

            string version = Assembly.GetExecutingAssembly().GetName().Version.ToString();
            this.lbl_Name.Text = string.Format(TitleMsg, version);

            #if DEBUG
            this.TopMost = false;
            #endif
        }

        #endregion Constructors

        #region Properties

        private bool IsAlive
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        private void ExecuteFadeIn()
        {
            this.SetOpacity(0);
            for (int i = 0; i < 10; i++)
            {
                this.Fading(0.1d);
                if (i < 9)
                    Thread.Sleep(100);
            }

            this.SetOpacity(1);
        }

        private void ExecuteFadeOut()
        {
            this.SetOpacity(1);
            for (int i = 0; i < 10; i++)
            {
                this.Fading(-0.1d);
                Thread.Sleep(100);
            }

            this.SetOpacity(0);
        }

        private void Fading(double opacity)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action<double>(value => this.Fading(value)), opacity);
            }
            else
            {
                this.Opacity += opacity;
            }
        }

        private void SetOpacity(double opacity)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action<double>(value => this.SetOpacity(value)), opacity);
            }
            else
            {
                this.Opacity = opacity;
            }
        }

        private void SplashScreen_Load(object sender, EventArgs e)
        {
            this.th_Animation.Start();
            this.th_Load.Start();
        }

        private void StepAnimation()
        {
            if (step == 50)
                step = 1;
            else
                step++;

            if (this.InvokeRequired)
            {
                //BUG: cannot access a disposed object exception was thrown here.
                //Impossible to reproduce that error. Keep an eye and add logs...
                this.Invoke(new Action(() => this.StepAnimation()));
            }
            else
            {
                this.pb_ProgressBar.Value = step;
            }
        }

        private void ThreadedAnimation()
        {
            this.Logger.Debug("ThreadedAnimation take the semaphore");
            this.semaphore.WaitOne();
            {
                this.ExecuteFadeIn();

                while (this.IsAlive)
                {
                    this.StepAnimation();
                    Thread.Sleep(100);
                }
            }
            this.ExecuteFadeOut();
            this.semaphore.Release();
            this.Logger.Debug("ThreadedAnimation released the semaphore");
        }

        private void ThreadLoad()
        {
            try
            {
                //StructureMap
                ObjectFactory.Initialize(e => e.UseDefaultStructureMapConfigFile = false);
                this.Logger.Info("StructureMap initialised.");

                //Translator
                //this.Logger.Info("Translator initialised.");

                //nHibernate
                DalFactory.Configurator.Configure();
                this.Logger.Info("Dal initialised.");
                this.IsAlive = false;

                this.Logger.Debug("ThreaLoad take the semaphore");
                this.semaphore.WaitOne();
                base.DialogResult = DialogResult.OK;
                this.semaphore.Release();
                this.Logger.Debug("ThreadLoad released the semaphore");
            }
            catch (Exception ex)
            {
                this.Logger.Error(ex.Message, ex);
            #if DEBUG
                MessageBoxes.Error(ex);
            #else
                MessageBoxes.Error("An unexpected error occured: {0}{1}{0} For more information, check the logs!"
                        .Translate(Environment.NewLine, ex.Message));
            #endif

                base.DialogResult = DialogResult.Cancel;
            }
            finally
            {
                this.IsAlive = false;
            }
        }

        #endregion Methods
    }
}