﻿namespace Phoenix.Model.Management
{
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Dal;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Translation;

    /// <summary>
    /// Manage drugs
    /// </summary>
    public class DrugModel : Manager<Drug>
    {
        #region Fields

        private IDrugTypeRepository drugTypeRepository = RepositoryFactory.Get<IDrugTypeRepository>();
        private IDrugInputPresenter presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DrugModel"/> class.
        /// </summary>
        /// <param name="presenter">The view.</param>
        public DrugModel(IDrugInputPresenter presenter)
            : base(RepositoryFactory.Get<IDrugRepository>())
        {
            this.presenter = presenter;
            this.presenter.Inserted += (sender, e) => this.Repository.Create(this.presenter.ItemToManage);
            this.presenter.Updated += (sender, e) => this.Repository.Update(this.presenter.ItemToManage);
            this.presenter.Deleted += (sender, e) => this.Delete(this.presenter.ItemToManage);
            this.presenter.Refreshed += (sender, e) =>
            {
                this.presenter.Types = this.drugTypeRepository.GetAllRows(OrderBy.ASC);
            };
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns>
        /// 	<c>True</c> if the data have passed the validation;
        /// otherwise <c>False</c>
        /// </returns>
        public override bool ValidateData()
        {
            this.presenter.ClearError();
            bool isValid = true;
            if (string.IsNullOrEmpty(this.presenter.ItemToManage.Title))
            {
                this.presenter.SetError(DrugFailedOn.NoName, "The drug doesn't have a name".Translate());
                isValid = false;
            }
            if (this.presenter.ItemToManage.Type == null || this.presenter.ItemToManage.Type.IsEmpty)
            {
                this.presenter.SetError(DrugFailedOn.NoType, "The drug doesn't have any type".Translate());
                isValid = false;
            }
            return isValid;
        }

        /// <summary>
        /// Process to execute when a delete is raised.
        /// </summary>
        /// <param name="item">The item to delete.</param>
        protected override void Delete(Drug item)
        {
            if (this.Repository.IsReferenced(this.presenter.ItemToManage))
            {
                this.presenter.SetError(DrugFailedOn.Referenced, "Impossible to delete this item, it is referenced somewhere else.");
            }
            else
            {
                base.Delete(item);
            }
        }

        #endregion Methods
    }
}