﻿namespace Phoenix.Model.Search
{
    using System;
    using System.Collections.Generic;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Dal;
    using Phoenix.Presenter.Interfaces.Searcher;

    /// <summary>
    /// Provide a Searcher with features focused on a Patient.
    /// </summary>    
    public abstract class PatientSearcher : Searcher<Patient>
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PatientSearcher"/> class.
        /// </summary>
        /// <param name="view">The presenter.</param>        
        protected PatientSearcher(IPatientSearcher view)
            : base(view)
        {
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Deletes the specified item from the repository.
        /// </summary>
        /// <param name="item">The item.</param>
        public override void Delete(Patient item)
        {
            Assert.IsTrue((item != null), "The value of the patient to delete can't be 'null'!");
            RepositoryFactory.Get<IPatientRepository>().Delete(item);
        }

        /// <summary>
        /// Start a search on a user demand.
        /// </summary>
        /// <param name="criterion">The criterion.</param>
        /// <returns>A list of patient result of the search.</returns>
        public override IList<Patient> ProcessSearch(object criterion)
        {
            Patient patient = criterion as Patient;
            Assert.IsNotNull(patient, "patient", "The criterion should be a 'Patient'");

            IList<Patient> result = RepositoryFactory.Get<IPatientRepository>().Search(patient);
            if (result.Count == 0) this.View.WarnOnZeroResult();
            return result;
        }

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns></returns>
        public bool ValidateData()
        {
            bool result = true;
            IPatientSearcher view = this.View as IPatientSearcher;

            Assert.IsType(this.View.Criterion, typeof(Patient));
            Patient patientCriterion = this.View.Criterion as Patient;

            DateTime dob = patientCriterion.DateOfBirth;
            DateTime ins = patientCriterion.InscriptionDate;

            if (!view.ValidateCriterions()) result = false;
            else if ((!dob.IsMinValue() && !ins.IsMinValue()) && (dob > ins))
            {
                view.InvalidateInscription("The inscription date can't be bigger than the date of birth");
                result = false;
            }

            return result;
        }

        #endregion Methods
    }
}