﻿namespace Phoenix.Model.Search
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Threading;

    using Phoenix.Helpers.Misc;
    using Phoenix.Presenter.Interfaces.Interfaces;

    /// <summary>
    /// Root class for any searcher.
    /// </summary>
    /// <typeparam name="T">The type the searcher searches</typeparam>
    public abstract class Searcher<T> : Presenter
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="Searcher&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="view">The view.</param>
        protected Searcher(ISearchBox<T> view)
            : base()
        {
            this.View = view;
            this.View.Searching += SearchedFired;
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Represent the presenter of the MVP model for a Seacher
        /// </summary>        
        protected ISearchBox<T> View
        {
            get;
            private set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Deletes the specified item from the repository.
        /// </summary>
        /// <param name="item">The item.</param>
        public abstract void Delete(T item);

        /// <summary>
        /// Start a search on a user demand.
        /// </summary>
        /// <param name="criterion">The criterion.</param>
        /// <returns></returns>
        public abstract IList<T> ProcessSearch(object criterion);

        /// <summary>
        /// Called when the user demands a search.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>        
        protected void SearchedFired(object sender, EventArgs<T> e)
        {
            var thread = new BackgroundWorker();
            thread.DoWork += (sender1, e1) =>
            {
                this.View.StartGuiWait();
                IReport report = Benchmark.GetReport();
                using (new Benchmark(report))
                {
                    var result = this.ProcessSearch(this.View.Criterion);
                    this.View.Fill(result);
                }
                this.Logger.DebugFormat("Search executed in {0} sec", report.ElapsedSeconds);
                this.View.EndGuiWait();
            };
            thread.RunWorkerAsync();
        }

        #endregion Methods
    }
}