﻿namespace Phoenix.Test.MedicalCardRelease
{
    using System;
    using System.Diagnostics;

    using NUnit.Framework;

    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Interfaces.Input;

    using Rhino.Mocks;
    using Rhino.Mocks.Interfaces;

    /// <summary>
    /// Contains the test for the macro manager
    /// </summary>
    [TestFixture]
    public class MacroTestSuite : TestSuite
    {
        #region Methods

        [Test]
        public void CanBindPatientToMacro()
        {
            string patientFirstName = "Luc";
            string patientLastName = "Skywalker";
            string textPattern = "Hello {0}. You have {1} years old!";
            DateTime dob = new DateTime(1978, 06, 02);
            int age = dob.GetAge();

            Patient patient = new Patient()
            {
                FirstName = patientFirstName,
                LastName = patientLastName,
                DateOfBirth = dob
            };

            string macroTitle = "Macro test";
            Macro macro = new Macro();

            macro.Text = string.Format(textPattern
                , Macro.GetTag(Macro.Predifined.PatientName)
                , Macro.GetTag(Macro.Predifined.PatientAge));
            macro.Title = macroTitle;
            macro.Bind(patient);

            Debug.WriteLine(macro.Text);

            Assert.AreEqual(string.Format(textPattern, patient, age)
                , macro.BindedText
                , "The binding of the macro didn't worked as expected!");
        }

        [Test]
        public void CanRaiseEvents()
        {
            #region Setup
            IEventRaiser raiseDelete, raiseAdd, raiseUpdate;
            IRepository<Macro> repository = this.Mocks.DynamicMock<IRepository<Macro>>();
            IMacroInputPresenter view = this.Mocks.DynamicMock<IMacroInputPresenter>();
            MacroModel presenter = new MacroModel(view, repository);

            Macro validMacro = new Macro()
            {
                Title = "Can Add Macro",
                Text = "Some text",
            };
            #endregion

            using (this.Mocks.Record())
            {
                SetupResult.For(view.ItemToManage).Return(validMacro);
                this.EventCounter = 0;

                view.Inserted += (sender, e) =>
                {
                    EventCounter++;
                };
                raiseAdd = LastCall.IgnoreArguments().GetEventRaiser();

                view.Deleted += (sender, e) =>
                {
                    EventCounter++;
                };
                raiseDelete = LastCall.IgnoreArguments().GetEventRaiser();

                view.Updated += (sender, e) =>
                {
                    EventCounter++;
                };
                raiseUpdate = LastCall.IgnoreArguments().GetEventRaiser();
            }
            raiseAdd.Raise(this, EventArgs.Empty);
            raiseDelete.Raise(this, EventArgs.Empty);
            raiseUpdate.Raise(this, EventArgs.Empty);

            Assert.AreEqual(3, EventCounter, string.Format(
                "Events ware fired '{0}' times but 3 times was expected!",
                EventCounter));
        }

        [Test]
        public void OnInvalidData()
        {
            #region Setup
            IRepository<Macro> repository = this.Mocks.DynamicMock<IRepository<Macro>>();
            IMacroInputPresenter view = this.Mocks.DynamicMock<IMacroInputPresenter>();

            MacroModel presenter = new MacroModel(view, repository);
            Macro invalidItem = new Macro();
            #endregion

            using (this.Mocks.Record())
            {
                SetupResult.For(view.ItemToManage).Return(invalidItem);
            }

            bool result = presenter.ValidateData();
            Assert.AreEqual(result, false, "The validation should fail!");
        }

        [Test]
        public void OnValidData()
        {
            #region Setup
            IMacroInputPresenter view = this.Mocks.DynamicMock<IMacroInputPresenter>();
            IRepository<Macro> repository = this.Mocks.DynamicMock<IRepository<Macro>>();

            MacroModel presenter = new MacroModel(view, repository);
            Macro validItem = new Macro()
            {
                Text = "Some Valid Text",
                Title = "Some Valid Title"
            };

            #endregion

            using (this.Mocks.Record())
            {
                SetupResult.For(view.ItemToManage).Return(validItem);
            }

            bool result = presenter.ValidateData();
            Assert.AreEqual(result, true, "The validation should succeed!");
        }

        [SetUp]
        public void Setup()
        {
            this.Mocks = new MockRepository();
            this.EventCounter = 0;
        }

        #endregion Methods
    }
}