﻿namespace Phoenix.View.Input
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Translation;

    /// <summary>
    /// Provides management of the keywords
    /// </summary>
    public partial class KeywordInput : TranslateableForm, IKeywordInputPresenter
    {
        #region Fields

        private KeywordGroup itemToManage = new KeywordGroup();
        private List<KeywordGroup> keywordGroups = new List<KeywordGroup>();
        private KeywordGroup newGroup = null;
        private KeywordModel presenter;
        private Usage usage = Usage.Display;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="KeywordInput"/> class.
        /// </summary>
        public KeywordInput()
        {
            InitializeComponent();
            this.TranslateForm();

            this.presenter = new KeywordModel(this);
            this.Logger = LogManager.GetLogger(typeof(KeywordInput));
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when a keyword group is deleted.
        /// </summary>
        public event EventHandler Deleted;

        /// <summary>
        /// Occurs when a keyword group is added.
        /// </summary>
        public event EventHandler Inserted;

        /// <summary>
        /// Occurs when keyword group is updated.
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public KeywordGroup ItemToManage
        {
            get
            {
                this.itemToManage.Color = this.lbl_Colour.BackColor;
                this.itemToManage.Title = this.tb_Title.Text;
                this.itemToManage.RawKeywords = this.tb_Keywords.Text;
                return this.itemToManage;
            }
            set
            {
                this.itemToManage = value;
                this.tb_Title.Text = value.Title;
                this.tb_Keywords.Text = value.RawKeywords;
                this.lbl_Colour.BackColor = value.Color;
            }
        }

        /// <summary>
        /// Gets or sets the list keyword groups.
        /// </summary>
        /// <value>The keyword groups.</value>
        public List<KeywordGroup> KeywordGroups
        {
            get
            {
                return this.keywordGroups;
            }
            set
            {
                this.keywordGroups = value;
                this.lv_Group.Items.Clear();
                foreach (var keyword in value)
                {
                    ListViewItem item = new ListViewItem(keyword.Title);
                    item.Tag = keyword;
                    this.lv_Group.Items.Add(item);
                }
            }
        }

        private KeywordGroup NewGroup
        {
            get
            {
                return (this.newGroup == null)
                    ? new KeywordGroup()
                    : this.newGroup;
            }
            set { this.newGroup = value; }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears the invalidation error.
        /// </summary>
        public void ClearError()
        {
            this.ep_Error.Clear();
        }

        /// <summary>
        /// Sets an error message to explain why data validation failed.
        /// </summary>
        /// <param name="failure"></param>
        /// <param name="errorMessage"></param>
        public void Invalidate(KeywordGroupFailedOn failure, string errorMessage)
        {
            Control control = new Control();
            switch (failure)
            {
                case KeywordGroupFailedOn.NoKeywords:
                    control = this.tb_Keywords;
                    break;
                case KeywordGroupFailedOn.NoTitle:
                    control = this.tb_Title;
                    break;
                default:
                    Assert.FailsOnEnumeration(failure);
                    break;
            }

            this.ep_Error.SetError(control, errorMessage);
        }

        private void ActivateContextMenu(bool enabled)
        {
            this.tsmi_DeleteGroup.Enabled = enabled;
            this.tsmi_UpdateGroup.Enabled = enabled;
        }

        private void Button_Click(object sender, EventArgs e)
        {
            if (sender == btn_OK)
            {
                if (!this.presenter.ValidateData()) return;

                switch (this.usage)
                {
                    case Usage.Insert:
                        this.Inserted.Raise(this);
                        this.ClearData();
                        break;
                    case Usage.Update:
                        this.Updated.Raise(this);
                        this.ClearData();
                        break;
                    case Usage.Delete:
                        this.Deleted.Raise(this);
                        this.ClearData();
                        break;
                    default:
                        Assert.FailsOnEnumeration(this.usage);
                        break;
                }
                this.EnableGui(false);
            }
            else if (sender == btn_Cancel)
            {
                this.Close();
            }
            else if (sender == btn_Colour)
            {
                DialogResult dr = this.cd_Colour.ShowDialog();
                if (dr != DialogResult.OK) return;

                this.lbl_Colour.BackColor = this.cd_Colour.Color;

            }
        }

        private void ClearData()
        {
            this.tb_Title.Text = string.Empty;
            this.tb_Keywords.Text = string.Empty;
            this.ItemToManage = new KeywordGroup();
        }

        private void CMenuGroup_Click(object sender, EventArgs e)
        {
            if (sender == this.tsmi_AddGroup)
            {
                this.usage = Usage.Insert;
                this.btn_OK.Text = "Add".Translate();
                this.EnableGui(true);
            }
            else if (sender == this.tsmi_DeleteGroup)
            {
                this.EnableGui(false);
                this.usage = Usage.Delete;
                this.btn_OK.Text = "Delete".Translate();
                this.btn_OK.Enabled = true;
                this.LoadSelectedGroup();
            }
            else if (sender == this.tsmi_UpdateGroup)
            {
                this.usage = Usage.Update;
                this.btn_OK.Text = "Update".Translate();
                this.EnableGui(true);
                this.LoadSelectedGroup();
            }
        }

        private void EnableGui(bool enabled)
        {
            this.tb_Title.Text = string.Empty;
            this.tb_Keywords.Text = string.Empty;

            this.btn_OK.Enabled = enabled;
            this.tb_Keywords.Enabled = enabled;
            this.tb_Title.Enabled = enabled;
            this.btn_Colour.Enabled = enabled;
            this.lbl_Colour.BackColor = Color.Black;

            if (!enabled)
                this.btn_OK.Text = "OK".Translate();
        }

        private void KeywordManagerBox_Load(object sender, EventArgs e)
        {
            this.presenter.RefreshData();
        }

        private void LoadSelectedGroup()
        {
            if (lv_Group.SelectedItems.Count <= 0) return;
            KeywordGroup group = lv_Group.SelectedItems[0].Tag as KeywordGroup;

            if (group == null) return;
            this.ItemToManage = group;
        }

        private void lv_Group_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                ListViewItem item = lv_Group.GetItemAt(e.X, e.Y);
                if (item != null)
                {
                    this.lv_Group.SelectedItems.Clear();
                    this.lv_Group.Items[item.Index].Selected = true;
                }
            }
        }

        private void lv_Group_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                Point clickedPoint = new Point(e.X, e.Y);
                ListViewItem clickedNode = lv_Group.GetItemAt(e.X, e.Y);

                if (clickedNode != null && clickedNode.Tag != null)
                {
                    // Convert from Tree coordinates to Screen coordinates
                    Point screenPoint = lv_Group.PointToScreen(clickedPoint);
                    // Convert from Screen coordinates to Form coordinates
                    Point formPoint = this.PointToClient(screenPoint);
                    // Show context menu
                    this.cms_Group.Show(this, formPoint);
                    this.ActivateContextMenu(true);
                }
                else
                {
                    this.ActivateContextMenu(false);
                }
            }
        }

        #endregion Methods
    }
}