﻿namespace Phoenix.View.Management
{
    using System;
    using System.Collections.Generic;
    using System.Drawing;
    using System.Windows.Forms;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Interfaces.Manager;
    using Phoenix.Translation;

    /// <summary>
    /// Medical card type manager
    /// </summary>
    public partial class MedicalCardTypeManagerBox : TranslateableForm, IMedicalCardTypeManagerPresenter
    {
        #region Fields

        MedicalCardType itemToManage = new MedicalCardType();
        private List<MedicalCardType> medicalCardTypes = new List<MedicalCardType>();
        private MedicalCardTypeModel presenter;
        private Usage usage;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="MedicalCardTypeManagerBox"/> class.
        /// </summary>
        public MedicalCardTypeManagerBox()
        {
            InitializeComponent();
            this.TranslateForm();
            this.EnableDisplay(false);

            this.presenter = new MedicalCardTypeModel(this);
            this.Usage = Usage.Insert;
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when a medical card type is deleted.
        /// </summary>
        public event EventHandler Deleted;

        /// <summary>
        /// Occurs when a medical card type is added.
        /// </summary>
        public event EventHandler Inserted;

        /// <summary>
        /// Occurs when the view is refreshed.
        /// </summary>
        public event EventHandler Refreshed;

        /// <summary>
        /// Occurs when medical card type isupdated.
        /// </summary>
        public event EventHandler Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the description of the manager.
        /// </summary>
        /// <value>The description.</value>
        public string Description
        {
            get { return "Medical card manager"; }
        }

        /// <summary>
        /// Gets the value of the icon tp show in the manager icon.
        /// </summary>
        /// <value>The icon.</value>
        public IconValue IconValue
        {
            get { return IconValue.None; }
        }

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public MedicalCardType ItemToManage
        {
            get
            {
                this.itemToManage.Remark = this.tb_Remarks.Text;
                this.itemToManage.Title = this.tb_Title.Text;
                return this.itemToManage;
            }
            set
            {
                this.itemToManage = value;
                this.tb_Remarks.Text = value.Remark;
                this.tb_Title.Text = value.Title;

            }
        }

        /// <summary>
        /// Gets or sets all the medical cards.
        /// </summary>
        /// <value>The medical cards.</value>
        public List<MedicalCardType> MedicalCardTypes
        {
            get
            {
                return this.medicalCardTypes;
            }
            set
            {
                this.medicalCardTypes = value;
                this.RefreshGui();
            }
        }

        /// <summary>
        /// Gets or sets the usage of this manager.
        /// </summary>
        /// <value>The usage.</value>
        public Usage Usage
        {
            get
            {
                return this.usage;
            }
            set
            {
                this.usage = value;
                switch (value)
                {
                    case Usage.Insert:
                        this.btn_OK.Text = "Insert".Translate();
                        break;
                    case Usage.Update:
                        this.btn_OK.Text = "Update".Translate();
                        break;
                    case Usage.Delete:
                        this.btn_OK.Text = "Delete".Translate();
                        break;
                    default:
                        this.btn_OK.Text = "OK";
                        break;
                }
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears previous error on validation failed.
        /// </summary>
        public void ClearError()
        {
            this.ep_Error.Clear();
        }

        /// <summary>
        /// Sets an error message to explain why data validation failed.
        /// </summary>
        /// <param name="failure"></param>
        /// <param name="errorMessage"></param>
        public void Invalidate(MedicalCardTypeFailedOn failure, string errorMessage)
        {
            Control control = new Control();
            switch (failure)
            {
                case MedicalCardTypeFailedOn.EmptyTitle:
                    control = this.tb_Title;
                    break;
                default:
                    Assert.FailsOnEnumeration(failure);
                    break;
            }
            this.ep_Error.SetError(control, errorMessage);
        }

        /// <summary>
        /// Called when the component has to be refreshed.
        /// </summary>
        public void RefreshData()
        {
            this.Refreshed.Raise(this);
        }

        /// <summary>
        /// Refreshes the data.
        /// </summary>
        public void RefreshGui()
        {
            this.lv_Groups.Items.Clear();
            foreach (var type in this.MedicalCardTypes)
                this.Insert(type);
        }

        private void ActivateAllContextMenu(bool enabled)
        {
            this.tsmi_Delete.Enabled = enabled;
            this.tsmi_Update.Enabled = enabled;
        }

        private void ActivateDisplay(bool enabled)
        {
            this.tb_Remarks.Enabled = enabled;
            this.tb_Title.Enabled = enabled;
        }

        private void btn_OK_Click(object sender, EventArgs e)
        {
            switch (this.Usage)
            {
                case Usage.Insert:
                    if (this.presenter.ValidateData())
                    {
                        this.Inserted.Raise(this);
                        this.RefreshGui();
                        this.EnableDisplay(false);
                    }
                    break;
                case Usage.Update:
                    if (this.presenter.ValidateData())
                    {
                        this.Updated.Raise(this);
                        this.RefreshGui();
                        this.EnableDisplay(false);
                    }
                    break;
                case Usage.Delete:
                    if (this.presenter.ValidateData())
                    {
                        this.Deleted.Raise(this);
                        this.RefreshGui();
                        this.EnableDisplay(false);
                    }
                    break;
                default:
                    Assert.FailsOnEnumeration(this.Usage);
                    break;
            }
        }

        private void EnableDisplay(bool enabled)
        {
            this.tb_Remarks.Enabled = enabled;
            this.tb_Title.Enabled = enabled;
        }

        private void Insert(MedicalCardType type)
        {
            ListViewItem item = new ListViewItem(type.Title);
            ListViewItem.ListViewSubItem subitem = new ListViewItem.ListViewSubItem(item, type.Title);
            item.SubItems.Add(subitem);
            item.Tag = type;

            this.lv_Groups.Items.Add(item);
        }

        private void lv_Groups_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.lv_Groups.SelectedItems.Count > 0
                && this.lv_Groups.SelectedItems[0].Tag is MedicalCardType)
            {
                this.SelectForUpdate();
            }
        }

        private void lv_Macros_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                ListViewItem item = this.lv_Groups.GetItemAt(e.X, e.Y);
                if (item != null)
                {
                    this.lv_Groups.SelectedItems.Clear();
                    this.lv_Groups.Items[item.Index].Selected = true;
                }
            }
        }

        private void lv_Macros_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                Point clickedPoint = new Point(e.X, e.Y);
                ListViewItem clickedNode = this.lv_Groups.GetItemAt(e.X, e.Y);

                if (clickedNode != null && clickedNode.Tag != null)
                {
                    this.ActivateAllContextMenu(true);
                    // Convert from Tree coordinates to Screen coordinates
                    Point screenPoint = this.lv_Groups.PointToScreen(clickedPoint);
                    // Convert from Screen coordinates to Form coordinates
                    Point formPoint = this.PointToClient(screenPoint);
                    // Show context menu
                    this.cms_Context.Show(this, formPoint);
                }
                else
                {
                    this.ActivateAllContextMenu(false);
                }
            }
        }

        private void MedicalCardTypeManagerBox_Load(object sender, EventArgs e)
        {
            this.Refreshed.Raise(this);
        }

        private void SelectForUpdate()
        {
            this.EnableDisplay(true);
            this.Usage = Phoenix.Presenter.Helpers.Usage.Update;
            this.ItemToManage = lv_Groups.SelectedItems[0].Tag as MedicalCardType;
            this.ActivateDisplay(true);
        }

        private void tsmi_Context_Click(object sender, EventArgs e)
        {
            if (sender == this.tsmi_Update)
            {
                SelectForUpdate();
            }
            else if (sender == this.tsmi_Delete)
            {
                this.EnableDisplay(true);
                this.Usage = Phoenix.Presenter.Helpers.Usage.Delete;
                this.ItemToManage = lv_Groups.SelectedItems[0].Tag as MedicalCardType;
                this.ActivateDisplay(false);
            }
            else
            {
                this.EnableDisplay(true);
                this.ItemToManage = new MedicalCardType();
                this.Usage = Phoenix.Presenter.Helpers.Usage.Insert;
            }
        }

        #endregion Methods
    }
}