﻿namespace Phoenix.View.Management
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Core.Management;
    using Phoenix.Presenter.Interfaces.Manager;

    /// <summary>
    /// Add new pathology in the pathologic history of a patient.
    /// </summary>
    public partial class PathologicEventCreator : Form, IPathologicEventCreatorPresenter
    {
        #region Fields

        private PathologicEventCreatorModel model;
        private PathologicEvent pathologicEvent;
        private PathologicEvent pathology;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PathologicEventCreator"/> class.
        /// </summary>
        public PathologicEventCreator()
        {
            InitializeComponent();
            this.model = new PathologicEventCreatorModel(this);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PathologicEventCreator"/> class.
        /// </summary>
        /// <param name="itemToManage">The item to manage.</param>
        public PathologicEventCreator(PathologicEvent itemToManage)
            : this()
        {
            this.ItemToManage = itemToManage;
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when GUI should be refreshed.
        /// </summary>
        public event EventHandler Refreshed;

        /// <summary>
        /// Occurs when a type is selected in the GUI.
        /// </summary>
        public event EventHandler<EventArgs<PathologyType>> TypeSelected;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public PathologicEvent ItemToManage
        {
            get
            {
                this.pathologicEvent.Pathology.Type = this.cb_PathologyType.SelectedItem as PathologyType;
                this.pathologicEvent.Pathology = this.cb_Pathology.SelectedItem as Pathology;
                this.pathologicEvent.StartDate = this.dtp_StartDate.Value;
                this.pathologicEvent.EndDate = this.dtp_EndDate.Value;
                this.pathologicEvent.Remark = this.tb_Remark.Text;
                return this.pathologicEvent;
            }
            set
            {
                if (value == null) return;
                this.pathologicEvent = value;
                this.cb_PathologyType.SelectedItem = this.pathologicEvent.Pathology.Type;
                this.cb_Pathology.SelectedItem = this.pathologicEvent.Pathology;
                this.dtp_StartDate.Value = this.pathologicEvent.StartDate;
                this.dtp_EndDate.Value = this.pathologicEvent.EndDate;
                this.tb_Remark.Text = this.pathologicEvent.Remark;
            }
        }

        /// <summary>
        /// Gets or sets the pathologies.
        /// </summary>
        /// <value>The pathologies.</value>
        public IEnumerable<Pathology> Pathologies
        {
            get
            {
                return this.cb_Pathology.DataSource as IEnumerable<Pathology>;
            }
            set
            {
                this.cb_Pathology.DataSource = null;
                this.cb_Pathology.DataSource = value;
                this.cb_Pathology.DisplayMember = "Title";
            }
        }

        /// <summary>
        /// Gets or sets the pathology types.
        /// </summary>
        /// <value>The pathology types.</value>
        public IEnumerable<PathologyType> PathologyTypes
        {
            get
            {
                return this.cb_PathologyType.DataSource as IEnumerable<PathologyType>;
            }
            set
            {
                this.cb_PathologyType.DataSource = null;
                this.cb_PathologyType.DataSource = value;
                this.cb_PathologyType.DisplayMember = "Title";
            }
        }

        private TimeSpan TimeSpan
        {
            get
            {
                return this.dtp_EndDate.Value - this.dtp_StartDate.Value;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears the errors.
        /// </summary>
        public void ClearErrors()
        {
            this.ep_Error.Clear();
        }

        /// <summary>
        /// Sets the error.
        /// </summary>
        /// <param name="failure">The failure.</param>
        /// <param name="message">The message.</param>
        public void SetError(PathologicEventFailedOn failure, string message)
        {
            Control control = new Control();
            switch (failure)
            {
                case PathologicEventFailedOn.WithoutPathology:
                    control = this.cb_Pathology;
                    break;
                case PathologicEventFailedOn.WithoutPathologyType:
                    control = this.cb_PathologyType;
                    break;
                case PathologicEventFailedOn.DateInFuture:
                case PathologicEventFailedOn.EndDateBeforeStartDate:
                    control = this.lbl_TimeSpan;
                    break;
                default:
                    Assert.FailsOnEnumeration(failure);
                    break;
            }
            this.ep_Error.SetError(control, message);
        }

        private bool AddPathologicEvent()
        {
            this.pathology = this.pathology ?? new PathologicEvent();

            pathology.StartDate = this.dtp_StartDate.Value;
            pathology.EndDate = this.dtp_EndDate.Value;
            pathology.Pathology = this.cb_Pathology.SelectedItem as Pathology;
            pathology.Remark = this.tb_Remark.Text;

            if (this.model.ValidateData(pathology))
            {
                this.ItemToManage = pathology;
                return true;
            }
            else
            {
                this.ItemToManage = null;
                return false;
            }
        }

        private void btn_Cancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
        }

        private void btn_OK_Click(object sender, EventArgs e)
        {
            if (this.AddPathologicEvent())
            {
                this.DialogResult = DialogResult.OK;
                this.Close();
            }
        }

        private void cb_PathologyType_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.TypeSelected.Raise(this, new EventArgs<PathologyType>(this.cb_PathologyType.SelectedItem as PathologyType));
        }

        private void dtp_ValueChanged(object sender, EventArgs e)
        {
            TimeSpan timeSpan = this.TimeSpan;
            this.lbl_TimeSpan.Text = string.Format("{0} day(s)", timeSpan.Days);
        }

        private void PathologicEventCreator_Load(object sender, EventArgs e)
        {
            this.Refreshed.Raise(this);
        }

        #endregion Methods
    }
}