﻿namespace Phoenix.View.Management
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Drawing;
    using System.Windows.Forms;

    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Interfaces.Manager;
    using Phoenix.Translation;

    using Savage.DotNetLibrary.ImageLibrary;

    /// <summary>
    /// Gui item to manage pictures of a patient
    /// </summary>
    public partial class PictureManagerBox : TranslateableForm, IPictureManagerPresenter
    {
        #region Fields

        private const int THUMB_SIZE = 100;

        private Patient patient = new Patient();
        private Picture picture = new Picture();
        private PictureModel presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PictureManagerBox"/> class.
        /// </summary>
        public PictureManagerBox()
        {
            InitializeComponent();
            this.presenter = new PictureModel(this);
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when a picture is deleted.
        /// </summary>
        public event EventHandler<EventArgs<Picture>> Deleted;

        /// <summary>
        /// Occurs when user wants to insert an item.
        /// </summary>
        public event EventHandler<EventArgs<Picture>> Inserted;

        /// <summary>
        /// [NOT USED] Occurs when the view is refreshed.        
        /// </summary>
        public event EventHandler Refreshed;

        /// <summary>
        /// Occurs when user wants to update an item.
        /// </summary>
        public event EventHandler<EventArgs<Picture>> Updated;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets the description of the manager.
        /// </summary>
        /// <value>The description.</value>
        public string Description
        {
            get { return "Picture"; }
        }

        /// <summary>
        /// Gets the value of the icon tp show in the manager icon.
        /// </summary>
        /// <value>The icon.</value>
        public IconValue IconValue
        {
            get { return IconValue.Picture; }
        }

        /// <summary>
        /// Gets the item this manager manages.
        /// </summary>
        /// <value>The managed item.</value>
        public Picture ItemToManage
        {
            get
            {
                return this.picture;
            }
            set
            {
                this.picture = value;
                this.pb_Image.Image = value.Image;
                this.tb_Remarks.Text = value.Remark;
            }
        }

        /// <summary>
        /// Gets or sets the patient to manage.
        /// </summary>
        /// <value>The patient.</value>
        public Patient Patient
        {
            get { return this.patient; }
            set
            {
                this.patient = value;
                this.RefreshGui();
            }
        }

        /// <summary>
        /// Gets or sets the usage of this manager.
        /// </summary>
        /// <value>The usage.</value>
        public Usage Usage
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Called when the component has to be refreshed.
        /// </summary>
        public void RefreshData()
        {
            this.Refreshed.Raise(this);
        }

        /// <summary>
        /// Refreshes the data.
        /// </summary>
        public void RefreshGui()
        {
            this.Text = "Pictures of {0}".Translate(this.patient.ToString());

            this.ClearThumbnails();

            BackgroundWorker worker = new BackgroundWorker();
            worker.WorkerReportsProgress = true;
            worker.DoWork += (sender, e) => this.CreateThumbnails(e);
            worker.RunWorkerCompleted += (sender, e) => this.DisplayImages(e);
            worker.RunWorkerAsync();

            this.Refreshed.Raise(this);
        }

        /// <summary>
        /// Closes this manager.
        /// </summary>
        public void ShutDown()
        {
            this.OnClosing(new CancelEventArgs());
            this.Close();
        }

        private void AddThumbnail(PictureBox thumb)
        {
            if (this.flp_Thumbnail.InvokeRequired)
            {
                this.flp_Thumbnail.Invoke((Action)delegate { this.AddThumbnail(thumb); });
            }
            else
            {
                this.flp_Thumbnail.Controls.Add(thumb);
            }
        }

        private void btn_Add_Click(object sender, EventArgs e)
        {
            DialogResult dr = this.ofd_Picture.ShowDialog();
            if (dr != DialogResult.OK) return;

            this.ItemToManage = new Picture();
            this.ItemToManage.Bitmap = Bitmap.FromFile(ofd_Picture.FileName).ToBytes();
            this.ItemToManage.Remark = this.tb_Remarks.Text;

            this.Inserted.Raise(this, new EventArgs<Picture>(this.ItemToManage));
            this.RefreshGui();
        }

        private void btn_Delete_Click(object sender, EventArgs e)
        {
            DialogResult dr = DialogResult = MessageBoxes.Question("Do you want to delete this picture?".Translate());
            if (dr == DialogResult.No) return;
            this.Deleted.Raise(this, new EventArgs<Picture>(this.ItemToManage));
            this.pb_Image.Image = null;
            this.RefreshGui();
        }

        private void btn_Update_Click(object sender, EventArgs e)
        {
            this.ItemToManage.Remark = this.tb_Remarks.Text;
            this.Updated.Raise(this, new EventArgs<Picture>(this.ItemToManage));
        }

        private void ClearThumbnails()
        {
            try
            {
                if (this.flp_Thumbnail.InvokeRequired)
                {
                    this.flp_Thumbnail.Invoke((Action)delegate { this.ClearThumbnails(); });
                }
                else
                {
                    this.flp_Thumbnail.Controls.Clear();
                }
            }
            catch (Exception ex)
            {
                Logger.Error(ex);
                throw;
            }
        }

        private void CreateThumbnails(DoWorkEventArgs e)
        {
            var imgThumbList = new List<KeyValuePair<Picture, Bitmap>>();
            foreach (var picture in this.patient.Pictures)
            {
                ImageUtility iu = new ImageUtility((Bitmap)picture.Image);

                int width, height;
                this.GetThumbnailSize(iu.Bitmap, out width, out height);
                Bitmap thumb = iu.CreateThumbnail(width, height);

                imgThumbList.Add(new KeyValuePair<Picture, Bitmap>(picture, thumb));
            }
            e.Result = imgThumbList;
        }

        private void DisplayImages(RunWorkerCompletedEventArgs e)
        {
            var imgThumbList = e.Result as List<KeyValuePair<Picture, Bitmap>>;
            foreach (var imgThumb in imgThumbList)
            {
                #region Create picture box
                PictureBox pb = new PictureBox();
                pb.Image = imgThumb.Value;
                pb.Width = this.flp_Thumbnail.Width - 30;
                pb.Height = THUMB_SIZE;
                pb.Tag = imgThumb.Key;
                pb.SizeMode = PictureBoxSizeMode.CenterImage;
                pb.BorderStyle = BorderStyle.FixedSingle;
                pb.Click += (sender1, e1) =>
                {
                    PictureBox pbb = sender1 as PictureBox;
                    if (pbb == null) return;
                    this.ItemToManage = pbb.Tag as Picture;
                };
                #endregion

                this.AddThumbnail(pb);
            }
        }

        private void GetThumbnailSize(Bitmap bitmap, out int width, out int height)
        {
            if (bitmap.Height > bitmap.Width)
            {
                height = THUMB_SIZE;
                width = bitmap.Width / (bitmap.Height / THUMB_SIZE);
            }
            else
            {
                width = THUMB_SIZE;
                height = bitmap.Height / (bitmap.Width / THUMB_SIZE);
            }
        }

        private void pb_Image_DoubleClick(object sender, EventArgs e)
        {
            PictureDisplay box = new PictureDisplay(this.pb_Image.Image);
            box.ShowDialog();
        }

        #endregion Methods
    }
}