﻿namespace Phoenix.View.Management
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Drawing;
    using System.Windows.Forms;

    using Phoenix.Helpers.Misc;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Core.Management;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Interfaces.Manager;

    using Savage.DotNetLibrary.ImageLibrary;

    public partial class PictureViewer : TranslateableForm, IPictureDisplayer
    {
        #region Fields

        private const int THUMB_SIZE = 100;

        private List<Picture> images = new List<Picture>();
        private PictureDisplayModel model;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PictureViewer"/> class.
        /// </summary>
        public PictureViewer()
        {
            InitializeComponent();
            this.model = new PictureDisplayModel(this);
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when all pictures are loading.
        /// </summary>
        public event EventHandler AllPicturesLoading;

        /// <summary>
        /// Occurs when displaying picture.
        /// </summary>
        public event EventHandler<EventArgs<long>> DisplayingPicture;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the displayed image.
        /// </summary>
        /// <value>The displayed image.</value>
        public Image DisplayedImage
        {
            get
            {
                return this.pb_Image.Image;
            }
            set
            {
                this.pb_Image.Image = value;
            }
        }

        /// <summary>
        /// Gets or sets the pictures.
        /// </summary>
        /// <value>The pictures.</value>
        public Picture[] Pictures
        {
            get
            {
                return this.images.ToArray();
            }
            set
            {
                this.images = new List<Picture>(value);
                this.Display();
            }
        }

        #endregion Properties

        #region Methods

        private void AddThumbnail(PictureBox thumb)
        {
            if (this.flp_Thumbnail.InvokeRequired)
            {
                this.flp_Thumbnail.Invoke((Action)delegate { this.AddThumbnail(thumb); });
            }
            else
            {
                this.flp_Thumbnail.Controls.Add(thumb);
            }
        }

        private void CreateThumbnails(DoWorkEventArgs e, BackgroundWorker worker)
        {
            int percentage = this.Pictures.Length;
            foreach (var picture in this.Pictures)
            {
                ImageUtility iu = new ImageUtility((Bitmap)picture.Image);

                int width, height;
                this.GetThumbnailSize(iu.Bitmap, out width, out height);

                Bitmap thumb = iu.CreateThumbnail(width, height);
                var imgThumb = new KeyValuePair<Image, long>(thumb, picture.ID);
                worker.ReportProgress(--percentage, imgThumb);
            }
        }

        private void Display()
        {
            var thread = new BackgroundWorker();
            thread.WorkerReportsProgress = true;
            thread.DoWork += (sender, e) => CreateThumbnails(e, thread);
            thread.ProgressChanged += (sender, e) => this.ProgressChanged(e);
            thread.RunWorkerCompleted += (sender, e) => AppContext.ClearGlobalMessage();
            thread.RunWorkerAsync();
        }

        private void GetThumbnailSize(Bitmap bitmap, out int width, out int height)
        {
            if (bitmap.Height > bitmap.Width)
            {
                height = THUMB_SIZE;
                width = bitmap.Width / (bitmap.Height / THUMB_SIZE);
            }
            else
            {
                width = THUMB_SIZE;
                height = bitmap.Height / (bitmap.Width / THUMB_SIZE);
            }
        }

        private void OnDisplayPicture(long id)
        {
            this.DisplayingPicture.Raise(this, new EventArgs<long>(id));
        }

        private void OnLoadPicture()
        {
            this.AllPicturesLoading.Raise(this);
        }

        private void pb_Image_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            var display = new PictureDisplay(this.pb_Image.Image);
            display.ShowDialog();
        }

        private void PictureViewer_Load(object sender, EventArgs e)
        {
            this.OnLoadPicture();
        }

        private void ProgressChanged(ProgressChangedEventArgs e)
        {
            var imgThumb = (KeyValuePair<Image, long>)e.UserState;

            #region Create picture box
            PictureBox pb = new PictureBox();
            pb.Image = imgThumb.Key;
            pb.Width = this.flp_Thumbnail.Width - 30;
            pb.Height = THUMB_SIZE;
            pb.SizeMode = PictureBoxSizeMode.Zoom;
            pb.BorderStyle = BorderStyle.FixedSingle;
            pb.Click += (sender, e1) => this.OnDisplayPicture(imgThumb.Value);
            #endregion

            this.AddThumbnail(pb);
            AppContext.SendGlobalMessage("Left to load: {0}", e.ProgressPercentage);
        }

        #endregion Methods
    }
}