﻿namespace Phoenix.Presenter.Search
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Forms;

    using log4net;

    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Presenter.Search.Criterion;
    using Phoenix.Translation;
    using Phoenix.View;
    using Phoenix.View.Configuration;
    using Phoenix.View.Helpers;

    /// <summary>
    /// Provides generic search box.
    /// </summary>
    public partial class SearchBox : TranslateableForm
    {
        #region Fields

        private const string ResultCount = "{0} item(s)";

        private int searchCount = 0;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="SearchBox"/> class.
        /// </summary>
        public SearchBox()
        {
            InitializeComponent();
            this.TranslateForm();

            Logger = LogManager.GetLogger(typeof(SearchBox));

            foreach (string item in Enum.GetNames(typeof(SearchBoxView)))
            {
                View view = (View)Enum.Parse(typeof(View), item);
                this.cb_Display.Items.Add(new DisplayType(view));
            }

            AppearanceConfiguration config = new AppearanceConfiguration();
            this.defaultView = config.SearchBoxView;
            this.cb_Display.SelectedIndex = (int)this.defaultView;
            this.lv_Result.View = View.Details;
            this.tssl_ResultCount.Text = Translator.Translate(this.tssl_ResultCount.Text, 0);

            this.lv_Result.View = (View)config.SearchBoxView;
        }

        #endregion Constructors

        #region Enumerations

        /// <summary>
        /// Specifies how list items are displayed in a SearchBox
        /// control.
        /// </summary>
        public enum SearchBoxView
        {
            /// <summary>
            /// Each item appears on a separate line with further information about each
            /// item arranged in columns. The left-most column contains a small icon and
            /// label, and subsequent columns contain sub items as specified by the application.
            /// A column displays a header which can display a caption for the column. The
            /// user can resize each column at run time.
            /// </summary>
            Details,
            /// <summary>
            /// Each item appears as a full-sized icon with the item label and subitem information
            /// to the right of it. The subitem information that appears is specified by
            /// the application. This view is available only on Windows XP and the Windows
            /// Server 2003 family. On earlier operating systems, this value is ignored and
            /// the System.Windows.Forms.ListView control displays in the System.Windows.Forms.View.LargeIcon
            /// view.
            /// </summary>
            Tile,
        }

        /// <summary>
        /// Represent the type of the item.
        /// </summary>
        protected enum ItemType
        {
            /// <summary>
            /// The item is a woman.
            /// </summary>
            Woman,
            /// <summary>
            /// The item is a man.
            /// </summary>
            Man,
            /// <summary>
            /// The item is a doctor.
            /// </summary>
            Doctor,
            /// <summary>
            /// The item is a drug
            /// </summary>
            Drug,
        }

        #endregion Enumerations

        #region Properties

        /// <summary>
        /// Gets or sets the criterion for the search.
        /// </summary>
        /// <value>The criterion.</value>
        public virtual object Criterion
        {
            get;
            protected set;
        }

        /// <summary>
        /// Gets or sets the default view to for Patient display.
        /// </summary>
        /// <value>The default view.</value>
        public SearchBoxView DefaultView
        {
            get
            {
                return this.defaultView;
            }
            set
            {
                this.defaultView = value;
                this.lv_Result.View = (View)Enum.Parse(typeof(View), this.defaultView.ToString());
                this.cb_Display.SelectedIndex = (int)this.DefaultView;
            }
        }

        /// <summary>
        /// Gets the item the user selected on double click or on click
        /// on the 'select' button.
        /// </summary>
        public object SelectedItem
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets or sets the location of the splitter, in pixels, from the left or top
        /// edge of the System.Windows.Forms.SplitContainer.
        /// </summary>
        /// <value>The splitter distance.</value>
        public int SplitterDistance
        {
            get
            {
                return this.sc_Split.SplitterDistance;
            }
            set
            {
                this.sc_Split.SplitterDistance = value;
            }
        }

        /// <summary>
        /// Gets the criterion collection.
        /// </summary>
        /// <value>The criterion collection.</value>
        protected CriterionControlCollection Criterions
        {
            get { return this.ccc_Criterions; }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Indicates the user the application has finished working.
        /// Typically hide the sandglass cursor show the default cursor
        /// </summary>
        public virtual void EndGuiWait()
        {
            if (this.InvokeRequired)
                this.Invoke((Action)delegate { this.EndGuiWait(); });
            else
                this.Cursor = Cursors.Default;
        }

        /// <summary>
        /// Indicates the user the application is working.
        /// Typically show the sandglass cursor
        /// </summary>
        public virtual void StartGuiWait()
        {
            if (this.InvokeRequired)
                this.Invoke((Action)delegate { this.StartGuiWait(); });
            else
                this.Cursor = Cursors.WaitCursor;
        }

        /// <summary>
        /// Validates the criterions.
        /// </summary>
        /// <returns></returns>
        public bool ValidateCriterions()
        {
            if (this.ccc_Criterions.ActivatedCriterionCount == 0)
            {
                PredifinedMsgBoxes.WarnOnNoCriterionActivated();
                return false;
            }
            if (!this.ccc_Criterions.ValidateCriterions())
            {
                PredifinedMsgBoxes.WarnInvalidValues();
                return false;
            }
            return true;
        }

        /// <summary>
        /// Warns the user when search return zero result.
        /// </summary>
        public void WarnOnZeroResult()
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { this.WarnOnZeroResult(); });

            }
            else
            {
                MessageBoxes.Exclamation("The search returned zero result!".Translate());
            }
        }

        /// <summary>
        /// Activates the progress bar.
        /// </summary>        
        protected void ActivateProgressBar()
        {
            this.SetProgressBar(true);
        }

        /// <summary>
        /// Adds a new list view column.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="title">The title.</param>
        protected void AddListViewColumn(string name, string title)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { this.AddListViewColumn(name, title); });
            }
            else
            {
                ColumnHeader columnHeader = new ColumnHeader();
                columnHeader.Name = name;
                columnHeader.Text = title;
                this.lv_Result.Columns.Add(columnHeader);
            }
        }

        /// <summary>
        /// Adds the Patient in the result list.
        /// </summary>
        /// <param name="item">The item to insert.</param>
        protected void AddListViewItem(ListViewItem item)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { this.AddListViewItem(item); });
            }
            else
            {
                this.lv_Result.Items.Add(item);
                this.tssl_ResultCount.Text = string.Format(ResultCount, ++this.searchCount);
            }
        }

        /// <summary>
        /// Adds the list of Patients in the result list.
        /// </summary>
        /// <param name="items">The items.</param>
        protected void AddListViewItems(List<ListViewItem> items)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { this.AddListViewItems(items); });
            }
            else
            {
                IReport report = Benchmark.GetReport();
                using (new Benchmark(report))
                {
                    this.lv_Result.SuspendLayout();
                    {
                        foreach (var item in items)
                        {
                            this.lv_Result.Items.Add(item);
                        }
                        this.tssl_ResultCount.Text = string.Format(ResultCount, items.Count);
                    }
                    this.lv_Result.ResumeLayout();
                }
                this.Logger.DebugFormat("Fill the result box in {0} sec", report.ElapsedSeconds);
            }
        }

        /// <summary>
        /// AResizes all the columns of the list view.
        /// </summary>
        protected void AutoResizeListView()
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { this.AutoResizeListView(); });
            }
            else
            {
                try
                {
                    if (this.lv_Result.View == View.Details)
                    {
                        this.lv_Result.SuspendLayout();
                        foreach (ColumnHeader column in this.lv_Result.Columns)
                        {
                            column.AutoResize(ColumnHeaderAutoResizeStyle.ColumnContent);
                        }
                    }
                }
                catch (InvalidOperationException ex)
                {
                    string msg = "Impossible to resize the column of the listview";
                    this.Logger.Error(msg, ex);
                    MessageBoxes.Error(msg.Translate());
                }
                finally
                {
                    this.lv_Result.ResumeLayout();
                }
            }
        }

        /// <summary>
        /// Clears all the Patient from the list view.
        /// </summary>
        protected void ClearItemsFromListView()
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { this.ClearItemsFromListView(); });
            }
            else
            {
                this.searchCount = 0;
                this.tssl_ResultCount.Text = string.Format(ResultCount, this.searchCount);
                this.lv_Result.Items.Clear();
            }
        }

        /// <summary>
        /// Deactivates the progress bar.
        /// </summary>
        protected void DeactivateProgressBar()
        {
            this.SetProgressBar(false);
        }

        /// <summary>
        /// Gets the group with the specified group name.
        /// If the group does not exist, it'll be added automatically.
        /// </summary>
        /// <param name="groupName">Name of the group.</param>
        /// <returns></returns>
        protected ListViewGroup GetGroup(string groupName)
        {
            if (this.lv_Result.InvokeRequired)
            {
                ListViewGroup result = null;
                this.lv_Result.Invoke((Action)delegate { result = this.GetGroup(groupName); });
                return result;
            }
            else
            {
                bool exist = false;

                foreach (ListViewGroup current in lv_Result.Groups)
                {
                    if (current.Header == groupName)
                    {
                        exist = true;
                        break;
                    }
                }
                if (!exist)
                {
                    ListViewGroup group = new ListViewGroup(groupName, groupName);
                    group.Header = groupName;
                    this.lv_Result.Groups.Add(group);
                }
                return this.lv_Result.Groups[groupName];
            }
        }

        /// <summary>
        /// Process to execute just after the parent form has executed the Load event
        /// </summary>
        protected virtual void LoadProcess()
        {
            /*By default it does nothing*/
        }

        /// <summary>
        /// Logs and display a message box when an exception occures on search.
        /// </summary>
        /// <param name="ex">The ex.</param>
        protected void LogAndDisplayOnSearchError(Exception ex)
        {
            string msg = string.Format("An error occured during search: {0}", ex.Message);
            this.Logger.Error(msg, ex);
            MessageBoxes.Error("Can't execute the search: {0}".Translate(ex.Message));
        }

        /// <summary>
        /// Logs and display a message box when an exception occures on selection.
        /// </summary>
        /// <param name="ex">The ex.</param>
        protected void LogAndDisplayOnSelectError(Exception ex)
        {
            string msg = "An error occured during the selection: {0}";
            this.Logger.Error(string.Format(msg, ex.Message), ex);
            MessageBoxes.Error(msg.Translate(ex.Message));
        }

        /// <summary>
        /// Called when the user asked the deletion of an item.
        /// </summary>
        protected virtual void OnDeletion()
        {
            string msg = "Please implement the method 'OnDeletion' in the child class!";
            this.Logger.Warn(msg);
            throw new NotImplementedException(msg);
        }

        /// <summary>
        /// Called when a Patient search is fired.
        /// </summary>
        protected virtual void OnSearch()
        {
            string msg = "Please implement the method 'OnSearch' in the child class!";
            this.Logger.Warn(msg);
            throw new NotImplementedException(msg);
        }

        /// <summary>
        /// Called when a item selection is fired.
        /// </summary>
        protected virtual void OnSelection()
        {
            string msg = "Please implement the method 'OnSelection' in the child class!";
            this.Logger.Warn(msg);
            throw new NotImplementedException(msg);
        }

        /// <summary>
        /// Sets the image of the specified type for the specified ListViewItem.
        /// </summary>
        /// <param name="itemType">Type of the item.</param>
        /// <param name="listViewItem">The list view item.</param>
        /// <exception cref="ArgumentException"/>
        protected void SetImage(ItemType itemType, ListViewItem listViewItem)
        {
            switch (itemType)
            {
                case ItemType.Drug:
                case ItemType.Woman:
                case ItemType.Man:
                case ItemType.Doctor:
                    listViewItem.ImageKey = itemType.ToString();
                    break;
                default:
                    string msg = string.Format("The type '{0}' is unknown: impossible to set the image for this item!");
                    this.Logger.Warn(msg);
                    throw new ArgumentException(msg);
            }
        }

        /// <summary>
        /// Sets the splitter distance.
        /// </summary>
        protected void SetSplitterDistance()
        {
            int height = 0;
            foreach (CriterionControl citerion in this.Criterions)
            {
                height += citerion.Height + 10;
            }
            this.SplitterDistance = height + 50;
        }

        /// <summary>
        /// Provide custom checking before processing click in the search button.
        /// If this methods returns false, no further process for the click on search.
        /// </summary>
        /// <returns><c>True</c> if the check succeed. Otherwise returns <c>False</c></returns>
        protected virtual bool ValidateData()
        {
            return true;
        }

        private void btn_Cancel_Click(object sender, EventArgs e)
        {
            base.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void btn_Search_Click(object sender, EventArgs e)
        {
            if (!ValidateData()) return;
            if (!ValidateCriterions()) return;

            this.OnSearch();
        }

        private void btn_Select_Click(object sender, EventArgs e)
        {
            this.SelectItem();
        }

        private void cb_Display_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox cb = sender as ComboBox;
            if (cb == null)
            {
                string msg = string.Format("Sender type is '{0}' but 'ComboBox' was expected!", sender.GetType().Name);
                this.Logger.Warn(msg);
                return;
            }

            DisplayType dType = cb.SelectedItem as DisplayType;
            if (dType == null)
            {
                string msg = string.Format("Sender type is '{0}' but 'ComboBox' was expected!", sender.GetType().Name);
                this.Logger.Warn(msg);
                return;
            }

            this.lv_Result.View = dType.View;
        }

        private void FocusOnFirstActivatedCriterion()
        {
            foreach (var criterion in this.Criterions)
            {
                if (criterion.IsActivated)
                {
                    criterion.SelectInput();
                    return;
                }
            }
        }

        private void lv_Result_DoubleClick(object sender, EventArgs e)
        {
            this.SelectItem();
        }

        private void SearchBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.Enter)
            {
                if (lv_Result.Items.Count == 0) return;

                this.SelectedItem = this.lv_Result.Items[0].Tag;
                this.OnSelection();
                base.DialogResult = DialogResult.OK;
                this.Logger.InfoFormat("The user has selected a {0} with 'Ctrl+Enter'", this.SelectedItem.GetType().Name);
            }
        }

        private void SearchBox_Load(object sender, EventArgs e)
        {
            this.FocusOnFirstActivatedCriterion();
            this.LoadProcess();
        }

        private void SelectItem()
        {
            if (this.lv_Result.SelectedItems.Count <= 0) return;

            this.SelectedItem = this.lv_Result.SelectedItems[0].Tag;
            this.OnSelection();
            base.DialogResult = DialogResult.OK;
            this.Logger.InfoFormat("The user has selected a {0}!", this.SelectedItem.GetType().Name);
            this.Close();
        }

        private void SetProgressBar(bool activate)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { this.SetProgressBar(activate); });
            }
            else
            {
                this.tspb_Progress.Style = (activate)
                                               ? ProgressBarStyle.Marquee
                                               : ProgressBarStyle.Blocks;
            }
        }

        #endregion Methods

        #region Nested Types

        /// <summary>
        /// Represent the way to display result.
        /// </summary>
        protected class DisplayType
        {
            #region Constructors

            /// <summary>
            /// Initializes a new instance of the <see cref="DisplayType"/> class.
            /// </summary>
            /// <param name="view">The view.</param>
            public DisplayType(View view)
            {
                this.View = view;
            }

            #endregion Constructors

            #region Properties

            /// <summary>
            /// Gets or sets the view.
            /// </summary>
            /// <value>The view.</value>
            public View View
            {
                get;
                private set;
            }

            #endregion Properties

            #region Methods

            /// <summary>
            /// Returns a <see cref="System.String"/> that represents this instance.
            /// </summary>
            /// <returns>
            /// A <see cref="System.String"/> that represents this instance.
            /// </returns>
            public override string ToString()
            {
                return this.View.ToString().Translate();
            }

            #endregion Methods
        }

        #endregion Nested Types
    }
}