﻿namespace Phoenix.Plugins.Host
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Reflection;

    using Phoenix.Plugins.Host.Exceptions;
    using Phoenix.Plugins.Host.Interfaces;

    public class PluginRepository : IEnumerable<IPlugable>
    {
        #region Fields
        private static object monitor= new object();
        private Dictionary<string, IPlugable> plugins = new Dictionary<string, IPlugable>();
        private static PluginRepository instance = null;
        private string assemblyPath = string.Empty;
        private IHost host = null;

        #endregion Fields

        #region Constructors

        private PluginRepository(IHost host, string assemblyPath)
        {
            this.assemblyPath = assemblyPath;
            this.host = host;
        }

        public static void Load(IHost host, string assembly)
        {
            lock (monitor)
            {
                if (instance == null)
                    instance = new PluginRepository(host, assembly);
            }
        }

        public static PluginRepository Instance
        {
            get
            {
                if (instance == null)
                    throw new RepositoryNotLoadedException();
                return instance;
            }
        }

        #endregion Constructors

        #region Indexers

        public IPlugable this[string name]
        {
            get
            {
                return this.plugins[name];
            }
        }

        #endregion Indexers

        #region Methods

        public void Configure(Action<IPlugable> configurator)
        {
            this.Configure();

            foreach (var plugin in this.plugins.Values)
            {
                configurator(plugin);

                IGuiPlugable e = plugin as IGuiPlugable;
                if (e != null)
                {
                    e.InsertInto(this.host.Container);
                }
            }
        }

        public void Configure()
        {
            Assembly asm = Assembly.LoadFrom(assemblyPath);
            Type[] types = asm.GetTypes();

            foreach (var type in types)
            {
                List<object > attributes = new List<object>(type.GetCustomAttributes(typeof(PlugableAttribute), false));

                if (attributes.Count <= 0) continue;

                IPlugable plugable = Activator.CreateInstance(type) as IPlugable;

                if (this.plugins.ContainsKey(plugable.Name))
                    throw new PluginAlreadyLoadedException(plugable);

                this.plugins.Add(plugable.Name, plugable);
            }

            if (this.plugins.Count <= 0)
                throw new NoPluginInAssemblyException();
        }

        /// <summary>
        /// Gets the enumerator.
        /// </summary>
        /// <returns></returns>
        public IEnumerator<IPlugable> GetEnumerator()
        {
            foreach (var plugin in this.plugins.Values)
            {
                yield return plugin;
            }
        }

        /// <summary>
        /// Returns an enumerator that iterates through a collection.
        /// </summary>
        /// <returns>
        /// An <see cref="T:System.Collections.IEnumerator"/> object that can be used to iterate through the collection.
        /// </returns>
        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        #endregion Methods
    }
}